#!/usr/bin/env python3
############################################################################################
# Satellite: EOS-Aqua                                       Sensor: MODIS
#            EOS-Terra                                      Sensor: MODIS
#------------------------------------------------------------------------
#
# Hilversum   , 2022/04/09       License GPL3          (c) Rob Alblas (Ernst Lobsiger)
#
#************************************************************************
############################################################################################

import sys
import os
import argparse
from glob import glob

from satpy import Scene, MultiScene
from pyorbital.orbital import Orbital
from datetime import datetime, timedelta

from satpy_common import pos,sat
from satpy_common import collect_files,get_files,select_files,create_scene
from satpy_common import Set_Gen,Set_Overlay,ofile_name
from satpy_common import set_lonlat,rm_substr,create_pic,magick_string
from satpy_common import set_orbit
from satpy_common import mkloc_ymd
from satpy_common import open_viewer

from satpy_settings import toodrp,datdrp
from satpy_areas import list_areas

# vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
# ********** ADAPT THESE PARAMETERS TO YOUR PERSONAL NEEDS ***********

def def_segdir(Date):
  Yea=Date[:4]
  Mon=Date[4:6]
  Day=Date[6:8]
  segdir = '/srv/rec_0/E1B-TPL-1' + '/' + Yea + '/' + Mon + '/' + Day

  return segdir

# ******** TOUCH THE CODE BELOW ONLY IF YOU KNOW WHAT YOU DO *********
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

############################################################################################
# Files: like thin_MYD021KM.A2022068.1440.061.2022068162056.NRT.hdf(Aqua)
#             thin_MOD021KM.A2022068.1440.061.2022068153903.NRT.hdf (Terra)
#             0123456789012345678901234
#
# Time-format: YYYYDOY.HHmm
def load_filetbl(files):
  ofiles=[]
  for fn in files:
    # y/doy ==> yyymmdd
    fpos=fn.rindex('/')+1
    dpos=fn.find('.A',fpos)+2
    YYYYDOY=str(fn[dpos:dpos+7])
    dt=datetime.strptime(YYYYDOY,'%Y%j')
    date=dt.strftime('%Y%m%d')
    tpos=fpos+23
    time=str(fn[tpos:tpos+4])

    row=[]
    row.append(date)
    row.append(time)
    row.append(fn)
    ofiles.append(row)
  return ofiles

############################################################################################
# pos=position center area
# dat=requested date (YYYYmmDD)
# day_pass=requested pass: day(True) or night(False)
# segdir is location raw files
# tlefil=location and name tle file
def Set_Sat_Aqua_Terra(fn_list,pos,dat,segdir,satid):
  isat=sat
  extended_day=True

  if (satid[0]=='A'):
    satname='Aqua'
  else:
    satname='Terra'
  isat.satname=satname
  isat.satname_st=satname
  isat.lenseg=180                                  # length of a segment (in seconds)
  isat.sensor='modis'

  if (satid[0]=='A'):
    pre_filename='thin_MYD021KM.A'
    isat.d1 = int(13. + 30./60. - pos.lon/15. - 5.5) # east time w.r.t. center area longitude
    isat.d2 = int(13. + 30./60. - pos.lon/15. + 6.5) # west time w.r.t. center area longitude
  else:
    pre_filename='thin_MOD021KM.A'
    isat.d1 = int(10. + 30./60. - pos.lon/15. - 5.5) # east time w.r.t. center area longitude
    isat.d2 = int(10. + 30./60. - pos.lon/15. + 6.5) # west time w.r.t. center area longitude
  isat.tsrvc = 'Basic'
  isat.rchan = 'E1B-TPL-1'

  isat.pos=pos

  # Collect files
  if (fn_list):
    (cf,pf,nf)=get_files(fn_list)
  else:
    sfrmt=segdir + '/' + pre_filename
    efrmt='*.hdf'
    tfrmt='.'
    (cf,pf,nf)=collect_files(sfrmt,dat,tfrmt,efrmt,True,extended_day)

  isat.cfiles=load_filetbl(cf)
  isat.pfiles=load_filetbl(pf)
  isat.nfiles=load_filetbl(nf)

  isat.reader='modis_l1b'
  isat.segdir=segdir

  isat.logo='NOAA_1200x1200.png'
  return isat

# Set temporary directory and image directory
def set_dirs(basename,datdrp,destdir):
  if (basename[0]=='A'):
    tmpdir = datdrp + '/EMCdata/tmpdirs/xaqua'
  else:
    tmpdir = datdrp + '/EMCdata/tmpdirs/xterra'

  if (destdir==''):
    imgdir = datdrp + '/EMCdata/images/EOS-'+basename
  else:
    imgdir = destdir
    if (imgdir[len(imgdir)-1]=='/'):
      imgdir = imgdir[:-1]

  return tmpdir,imgdir

# Command line parsing
usage1 = 'Usage: ' + sys.argv[0] + ' -t YYYYmmDDHHMM  [-sat RSS|IOC|1|2|3|4] [-src sourcedir] [-dst destinationdir] [-area <area>] -composite <composite> [-e <area|composite>]'

parser = argparse.ArgumentParser('AquaTerra')
parser.add_argument('-t'        ,default='', help="time: YYYYmmDD")
parser.add_argument('-sat'      ,default='AD', help="A[D|N]] | T[D|N]: A=Aqua, T=Terra, D=Day, N=Night")
parser.add_argument('-src'      ,default='', help="source dir., MUST be absolute path!")
parser.add_argument('-dst'      ,default='', help="destination dir., MUST be absolute path!")
parser.add_argument('-sp'       ,nargs='?',const=True,default=False, help="stacked pass")
parser.add_argument('-fl'       ,default='', help="file with list of files")
parser.add_argument('-o'        ,default='', help="output file")
parser.add_argument('-area'     ,default='', help="area, 'list' gives a list")
parser.add_argument('-composite',default='', help="composite, 'list' gives a list")
parser.add_argument('-tle'      ,default='', help="tle file, must contain tle elements of used satellite")
parser.add_argument('-histtle'  ,nargs='?',const=True,default=False, help="get historical tle from spacetrack")
parser.add_argument('-v'        ,nargs='?',const=1,default=0, help="start viewer; -v <vprog>: specify viewer")
parser.add_argument('-dry'      ,nargs='?',const=1,default=0, help="dry run; -dry f: show used files only")
parser.add_argument('-nof'      ,nargs='?',const=1,default=0, help="don't overwrite")

args = parser.parse_args()
Dat=args.t
asat=args.sat
src=args.src
dst=args.dst
fn_list=args.fl
ofile=args.o
area=args.area
composite=args.composite
do_stack=args.sp
tlefil=args.tle
use_histtle=args.histtle
view=args.v
dry=args.dry
nof=args.nof

if (area=='list'):
  list_areas()
  sys.exit()

if (Dat=='') and (fn_list==''):
  sys.exit('Error: No -t or -fl specified.')

# asat=AD, AN, TD, TN
if ((len(asat) > 1) and (asat[1]=='N')):
  day_pass=False
else:
  day_pass=True

# A(qua) or T(erra)
if (len(asat) > 0):
  satid=asat[0]
else:
  satid='A'

if (tlefil == ''):
  if not use_histtle:
    tlefil = toodrp + '/EMCtools/pppconfig/my_TLE_file.txt'
print("Used tle="+tlefil)

if (composite==''):
  if (day_pass):
    composite = 'overview'
  else:
    composite = 'overview'

# Choose your area
# Area definition must consider above coords and range for best results
if (area==''):
  area='westminster'

# Center coords (° North and ° East) and range ° of interest
center_area=set_lonlat(area)
genpar=Set_Gen(day_pass,composite,area,dry,view)
Set_Overlay(genpar,ADDgrid=True,ADDcoasts=True,ADDborders=True,ADDrivers=True,ADDpoints=True)

if ((src!='') and (src[len(src)-1]=='/')):
  src = src[:-1]

if (src==''):
  segdir=def_segdir(Dat)
else:
  segdir = src

# try to search for YYYY/mm/DD location, extend segdir if exists 
#   (asume files are saved that way)
segdir=mkloc_ymd(segdir,Dat[:4],Dat[4:6],Dat[6:8])

curdir=os.getcwd()
satpar=Set_Sat_Aqua_Terra(fn_list,center_area,Dat,segdir,satid)
if not dry:
  satpar.orb=set_orbit(satpar,Dat,use_histtle,tlefil)
(tmpdir,imgdir)=set_dirs(satpar.satname,datdrp,dst)

#######################################################################################
# Dir must exist.
# Goto tmpdir, so tmp-files will be dropped there
try:
   os.chdir(tmpdir)
except:
   sys.exit('Cannot change to tmp directory ' + str(tmpdir))

if (dry!=1):
  (gf,gt)=select_files(satpar,genpar)

os.chdir(curdir)

ofile=ofile_name(satpar,genpar,ofile,Dat)
if nof:
  if os.path.isfile(imgdir+'/'+ofile):
    print('already exists: '+imgdir+'/'+ofile)
    if (view):
      open_viewer(genpar.viewer,imgdir+'/'+ofile)
    quit()

if dry:
  print('---------------Dry run---------------')
  print("Satellite       : " + asat)
  print("Date            : " + Dat)
  print("Area            : " + area)
  print("Composite       : " + composite)
  print("Source loc.     : " + segdir)
  print("Destination loc.: " + imgdir)
  print("temp. loc.      : " + tmpdir)
  print("TLE file        : " + tlefil)
  print("Use hist. tle   : " + str(use_histtle))
  print("File list       : " + fn_list)
  print("Stacking        : " + str(do_stack))
  print("Output file     : " + ofile)
  print("Viewer          : " + genpar.viewer)
  if (dry!='f'):
    sys.exit('Stop, dry mode')

print("Area     : " + area)
print("Composite: " + composite)

if (len(gf)==0):
   sys.exit('No files in area range')
print('Files in area range: ' + str(len(gf)))

#Next must be done in tmpdir to put temp. pic there (still in tmpdir, but anyway...)
os.chdir(tmpdir)
new_scn=create_scene(satpar,genpar,gf,gt,composite,do_stack,datdrp)

try: # This is for real (multichannel) composites
    layers, height, width = new_scn[composite].shape
except: # In case of a single channel 'composite'
    height, width = new_scn[composite].shape


os.chdir(curdir)
os.system(magick_string(satpar,genpar,tmpdir,datdrp+'/EMCdata/logos',height,imgdir+'/'+ofile))
print('Generated: ' + imgdir+'/'+ofile)

if (view):
  open_viewer(genpar.viewer,imgdir+'/'+ofile)
