#!/usr/bin/env python3
############################################################################################
# Satellite: NOAA-20    JPSS (Joint Polar Satellite System)        sensor viirs M-Bands
#            Suomi-NPP  JPSS (Joint Polar Satellite System)        sensor viirs M-Bands
#------------------------------------------------------------------------
#
# Hilversum   , 2022/04/09       License GPL3          (c) Rob Alblas (Ernst Lobsiger)
#
#************************************************************************
############################################################################################

import sys
import os
import argparse
from glob import glob

from satpy import Scene, MultiScene
from pyorbital.orbital import Orbital
from datetime import datetime, timedelta

from satpy_common import pos,sat
from satpy_common import collect_files,get_files,select_files,create_scene
from satpy_common import Set_Gen,Set_Overlay,ofile_name
from satpy_common import set_lonlat,rm_substr,create_pic,magick_string
from satpy_common import set_orbit
from satpy_common import mkloc_ymd
from satpy_common import open_viewer

from satpy_settings import toodrp,datdrp
from satpy_areas import list_areas

# vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
# ********** ADAPT THESE PARAMETERS TO YOUR PERSONAL NEEDS ***********

def def_segdir(nsatid,Date):
  Yea=Date[:4]
  Mon=Date[4:6]
  Day=Date[6:8]
  segdir=''
  if (nsatid=='S'):
    segdir = '/srv/rec_0/NPP-2' + '/' + Yea + '/' + Mon + '/' + Day
  if (nsatid=='N'):
    segdir = '/srv/rec_1/E1H-RDS-1' + '/' + Yea + '/' + Mon + '/' + Day

  return segdir

# ******** TOUCH THE CODE BELOW ONLY IF YOU KNOW WHAT YOU DO *********
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

############################################################################################
# Files: like SVMC_j01_d20190411_t0224358_e0226003_b07218_c20190411023355000121_eum_ops.h5    (NOAA20 day)
#             SVDNBC_j01_d20190411_t0424054_e0425281_b07219_c20190411043053000049_eum_ops.h5  (NOAA20 night)
#             SVMC_npp_d20190326_t0147221_e0148463_b38388_c20190326015546000177_eum_ops.h5    (Suomi day)
#             SVDNBC_npp_d20190325_t0211594_e0213236_b38373_c20190325022115000637_eum_ops.h5  (Suomi hight)
#             0123456789012345678901234
def load_filetbl(files):
  ofiles=[]
  for fn in files:
    fpos=fn.rindex('/')+1
    dpos=fn.find('_d',fpos+1)+2
    date=str(fn[dpos:dpos+8])
    tpos=fn.find('_t',fpos)+2
    time=str(fn[tpos:tpos+6])

    row=[]
    row.append(date)
    row.append(time)
    row.append(fn)
    ofiles.append(row)
  return ofiles

############################################################################################
# pos=position center area
# dat=requested date (YYYYmmDD)
# day_pass=requested pass: day(True) or night(False)
# segdir is location raw files
# tlefil=location and name tle file
def Set_Sat_NOAA20(fn_list,pos,dat,day_pass,segdir,satid):
  isat=sat
  extended_day=True
  if (satid[0]=='S'):
    satname='SUOMI NPP'
    isat.satname_st='NPP'
  else:
    satname='NOAA '
    if (len(asat) > 0):
      satname=satname+asat[1:3] # asat[1:3]='20', '21'?
    else:
      satname=satname+'20'
    isat.satname_st=satname

  isat.satname=satname.replace(' ','-')  
  isat.lenseg=86                                   # length of a segment (in seconds)
  isat.sensor='viirs'
  if (satname[0]=='S'):                            # Suomi
    if (day_pass):
      isat.pre_filename='SVMC_npp_d'                    # Day
    else:
      isat.pre_filename='SV*C_npp_d'                    # Night
    isat.d1 = int(13. + 25./60. - pos.lon/15. - 5.5) # east time w.r.t. center area longitude
    isat.d2 = int(13. + 25./60. - pos.lon/15. + 6.5) # west time w.r.t. center area longitude
    isat.tsrvc = 'HVS-1'
    isat.rchan = 'E1H-RDS-1'
  elif (satname[0]=='N'):
    if (day_pass):
      isat.pre_filename='SVMC_j01_d'                    # Day
    else:
      isat.pre_filename='SV*C_j01_d'                    # Night
    isat.d1 = int(13. + 30./60. - pos.lon/15. - 5.5)
    isat.d2 = int(13. + 30./60. - pos.lon/15. + 6.5)
    isat.tsrvc = 'Basic'
    isat.rchan = 'NPP-2'
  else:
    sys.exit('Unknown satellite: ' + satname)

  isat.d1=isat.d1%24
  isat.d2=isat.d2%24

  isat.pos=pos
 
  # Collect files
  if (fn_list):
    (cf,pf,nf)=get_files(fn_list)
  else:
    sfrmt=segdir + '/' + isat.pre_filename
    efrmt='*eum_ops.h5'
    tfrmt='_t'
    (cf,pf,nf)=collect_files(sfrmt,dat,tfrmt,efrmt,False,extended_day)

  isat.cfiles=load_filetbl(cf)
  isat.pfiles=load_filetbl(pf)
  isat.nfiles=load_filetbl(nf)

  isat.reader='viirs_compact'
  isat.segdir=segdir
  isat.logo='NOAA_1200x1200.png'
  return isat

# Set temporary directory and image directory
def set_dirs(basename,datdrp,destdir):
  if (basename[0]=='S'):
    tmpdir = datdrp + '/EMCdata/tmpdirs/xsnpp'
  else:
    tmpdir = datdrp + '/EMCdata/tmpdirs/x'+rm_substr(basename.lower(),'-')

  if (destdir==''):
    if (basename[:4]=='NOAA'):
      imgdir = datdrp + '/EMCdata/images/'+satpar.satname
    else:
      imgdir = datdrp + '/EMCdata/images/'+satpar.satname.capitalize()
      if (imgdir[len(imgdir)-3:] == 'npp'):
        imgdir = imgdir.rstrip('npp')
        imgdir = imgdir+"NPP"
  else:
    imgdir = destdir
    if (imgdir[len(imgdir)-1]=='/'):
      imgdir = imgdir[:-1]

  return tmpdir,imgdir

# Command line parsing
usage1 = 'Usage: ' + sys.argv[0] + ' -t YYYYmmDDHHMM  [-sat RSS|IOC|1|2|3|4] [-src sourcedir] [-dst destinationdir] [-area <area>] -composite <composite> [-e <area|composite>]'

parser = argparse.ArgumentParser('NOAA')
parser.add_argument('-t'        ,default='00', help="time: YYYYmmDD")
parser.add_argument('-sat'      ,default='N20D', help="N20|SUO [D|N]]: D=Day, N=Night")
parser.add_argument('-src'      ,default='', help="source dir., MUST be absolute path!")
parser.add_argument('-dst'      ,default='', help="destination dir., MUST be absolute path!")
parser.add_argument('-sp'       ,nargs='?',const=True,default=False, help="stacked pass")
parser.add_argument('-fl'       ,default='', help="file with list of files")
parser.add_argument('-o'        ,default='', help="output file")
parser.add_argument('-area'     ,default='', help="area, 'list' gives a list")
parser.add_argument('-composite',default='', help="composite, 'list' gives a list")
parser.add_argument('-tle'      ,default='', help="tle file, must contain tle elements of used satellite")
parser.add_argument('-histtle'  ,nargs='?',const=True,default=False, help="get historical tle from spacetrack")
parser.add_argument('-v'        ,nargs='?',const=1,default=0, help="start viewer; -v <vprog>: specify viewer")
parser.add_argument('-dry'      ,nargs='?',const=1,default=0, help="dry run; -dry f: show used files only")
parser.add_argument('-nof'      ,nargs='?',const=1,default=0, help="don't overwrite")

args = parser.parse_args()
Dat=args.t
asat=args.sat
src=args.src
dst=args.dst
fn_list=args.fl
ofile=args.o
area=args.area
composite=args.composite
do_stack=args.sp
tlefil=args.tle
use_histtle=args.histtle
view=args.v
dry=args.dry
nof=args.nof

if (area=='list'):
  list_areas()
  sys.exit()

if (Dat=='') and (fn_list==''):
  sys.exit('Error: No -t or -fl specified.')

# asat=N20D, N20N, SUOD, SUON
if ((len(asat) > 3) and (asat[3]=='N')):
  day_pass=False
else:
  day_pass=True

if (len(asat) > 0):
  satid=asat[0]
else:
  satid='N'

if (tlefil == ''):
  if not use_histtle:
    tlefil = toodrp + '/EMCtools/pppconfig/my_TLE_file.txt'
print("Used tle="+tlefil)

if (composite==''):
  if (day_pass):
    composite = 'false_color'
  else:
    composite = 'night_overview'

# Choose your area
# Area definition must consider above coords and range for best results
if (area==''):
  area='westminster'

# Center coords (° North and ° East) and range ° of interest
center_area=set_lonlat(area)
genpar=Set_Gen(day_pass,composite,area,dry,view)
Set_Overlay(genpar,ADDgrid=True,ADDcoasts=True,ADDborders=True,ADDrivers=True,ADDpoints=True)

if ((src!='') and (src[len(src)-1]=='/')):
  src = src[:-1]

if (src==''):
  segdir=def_segdir(satid,Dat)
else:
  segdir = src

# try to search for YYYY/mm/DD location, extend segdir if exists 
#   (asume files are saved that way)
segdir=mkloc_ymd(segdir,Dat[:4],Dat[4:6],Dat[6:8])

curdir=os.getcwd()
satpar=Set_Sat_NOAA20(fn_list,center_area,Dat,day_pass,segdir,satid)
if not dry:
  satpar.orb=set_orbit(satpar,Dat,use_histtle,tlefil)
(tmpdir,imgdir)=set_dirs(satpar.satname,datdrp,dst)

#######################################################################################

# Dir must exist.
# Goto tmpdir, so tmp-files will be dropped there
try:
   os.chdir(tmpdir)
except:
   sys.exit('Cannot change to tmp directory ' + str(tmpdir))

if (dry!=1):
  (gf,gt)=select_files(satpar,genpar)

os.chdir(curdir)

ofile=ofile_name(satpar,genpar,ofile,Dat)
if nof:
  if os.path.isfile(imgdir+'/'+ofile):
    print('already exists: '+imgdir+'/'+ofile)
    if (view):
      open_viewer(genpar.viewer,imgdir+'/'+ofile)
    quit()

if dry:
  print('---------------Dry run---------------')
  print("Satellite       : " + asat)
  print("Date            : " + Dat)
  print("Area            : " + area)
  print("Composite       : " + composite)
  print("Source loc.     : " + segdir)
  print("Destination loc.: " + imgdir)
  print("temp. loc.      : " + tmpdir)
  print("TLE file        : " + tlefil)
  print("Use hist. tle   : " + str(use_histtle))
  print("File list       : " + fn_list)
  print("Stacking        : " + str(do_stack))
  print("Output file     : " + ofile)
  print("Viewer          : " + genpar.viewer)
  if (dry!='f'):
    sys.exit('Stop, dry mode')

print("Area     : " + area)
print("Composite: " + composite)

if (len(gf)==0):
   sys.exit('No files in area range')
print('Files in area range: ' + str(len(gf)))

#Next must be done in tmpdir to put temp. pic there (still in tmpdir, but anyway...)
os.chdir(tmpdir)
new_scn=create_scene(satpar,genpar,gf,gt,composite,do_stack,datdrp)

try: # This is for real (multichannel) composites
    layers, height, width = new_scn[composite].shape
except: # In case of a single channel 'composite'
    height, width = new_scn[composite].shape

os.chdir(curdir)
os.system(magick_string(satpar,genpar,tmpdir,datdrp+'/EMCdata/logos',height,imgdir+'/'+ofile))
print('Generated: ' + imgdir+'/'+ofile)

if (view):
  open_viewer(genpar.viewer,imgdir+'/'+ofile)
